@@ -55,6 +55,11 @@ class Renderer(object):
     def template_names(self):
         return self._env.list_templates(filter_func=self.is_template)
 
+    @property
+    def templates(self):
+        for template_name in self.template_names:
+            yield self.get_template(template_name)
+
     def get_template(self, template_name):
         """Get a Template object from the environment.
 
@@ -144,7 +149,7 @@ class Renderer(object):
             if not os.path.exists(file_dirpath):
                 os.makedirs(file_dirpath)
 
-    def render_template(self, template, context):
+    def render_template(self, template, context=None):
         """Render a template.
 
         If a Rule matching the template is found, the rendering task is
@@ -155,6 +160,9 @@ class Renderer(object):
         """
         self.logger.info("Rendering %s..." % template.name)
 
+        if context is None:
+            context = self.get_context(template)
+
         try:
             rule = self.get_rule(template.name)
         except ValueError:
@@ -164,42 +172,76 @@ class Renderer(object):
         else:
             rule(self, template, **context)
 
-    def render_templates(self):
-        """Render each of the templates."""
-        for template_name in self.template_names:
-            template = self.get_template(template_name)
-            context = self.get_context(template)
-            self.render_template(template, context)
+    def render_templates(self, templates):
+        """Render a collection of templates.
+        
+        :param templates: a collection of Templates
+        """
+        for template in templates:
+            self.render_template(template)
 
-    def _watch(self):
-        """Watch and reload templates."""
-        import easywatch
-
-        self.logger.info("Watching '%s' for changes..." %
-                          self.searchpath)
-        self.logger.info("Press Ctrl+C to stop.")
-
-        def handler(event_type, src_path):
-            filename = os.path.relpath(src_path, self.searchpath)
-            if event_type == "modified":
-                if src_path.startswith(self.searchpath):
-                    if self.is_partial(filename):
-                        self.render_templates()
-                    elif self.is_template(filename):
-                        template = self.get_template(filename)
-                        context = self.get_context(template)
-                        self.render_template(filename)
-        easywatch.watch(self.searchpath, handler)
+    def get_dependencies(self, filename):
+        """Get every file that depends on a file.
+        
+        :param filename: the name of the file to find dependencies of
+        """
+        # TODO: Address partial dependencies more efficiently
+        if self.is_partial(filename):
+            return self.templates
+        elif self.is_template(filename):
+            return [self.get_template(filename)]
+        else:
+            return []
 
     def run(self, use_reloader=False):
         """Run the renderer.
 
         :param use_reloader: if given, reload templates on modification
         """
-        self.render_templates()
+        self.render_templates(self.templates)
 
         if use_reloader:
-            self._watch()
+            self.logger.info("Watching '%s' for changes..." %
+                              self.searchpath)
+            self.logger.info("Press Ctrl+C to stop.")
+            Reloader(self).watch()
+
+    def __repr__(self):
+        return "Renderer('%s', '%s')" % (self.searchpath, self.outpath)
+
+
+class Reloader(object):
+    """
+    Watches ``renderer.searchpath`` for changes and re-renders any changed
+    Templates.
+
+    :param renderer: A Renderer object, used to re-render templates.
+    """
+    def __init__(self, renderer):
+        self.renderer = renderer
+
+    @property
+    def searchpath(self):
+        return self.renderer.searchpath
+
+    def should_handle(self, event_type, filename):
+        """Check if an event should be handled.
+        
+        An event should be handled if a file in the searchpath was modified."""
+        return (event_type == "modified"
+                and filename.startswith(self.searchpath))
+
+    def event_handler(self, event_type, src_path):
+        """Re-render templates if they are modified."""
+        filename = os.path.relpath(src_path, self.searchpath)
+        if self.should_handle(event_type, filename):
+            templates = self.renderer.get_dependencies(filename)
+            self.renderer.render_templates(templates)
+
+    def watch(self):
+        """Watch and reload templates."""
+        import easywatch
+        easywatch.watch(self.searchpath, self.event_handler)
 
 
 def make_renderer(searchpath="templates",
